﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3

//#include <Graph Utility Procs>

// v 2.11b1

#pragma version = 2.11
#pragma IgorVersion = 8

#pragma ModuleName=imgT_Main

#include "Image Tools Controls"
#include "Image Tools Load"
#include "Image Tools Calibrate"
#include "Image Tools Process"

Static StrConstant k_version = "2.11"
Static Constant k_version = 2.11

// uncomment to debug
//#define DEBUG

#ifndef DEBUG
#pragma hide=1
#endif

// ***
// VARIABLES

// package definitions
// folders
Static StrConstant k_packageFolder = "imgT"
Static StrConstant k_histfolder = "imgT_hist"
Static StrConstant k_cpfolder = "imgT_cprint"
Static StrConstant k_fullpackageFolder = "root:Packages:imgT"

// windows
Static StrConstant k_imgDisplay = "imgT_display"
Static StrConstant k_histPanel = "Histograms"
Static StrConstant k_histGraphs = "imgT_histograms"
Static StrConstant k_panelName = "imgT_panel"
Static StrConstant k_fullhpanel = "imgT_display#Histograms"
Static StrConstant k_fullhgraphs = "imgT_display#Histograms#imgT_histograms"
Static StrConstant k_fullpanel = "imgT_display#imgT_panel"

// panel and graph dimensions
Static Constant pleft = 500
Static Constant ptop = 50
Static Constant pwidth = 415
Static Constant pheight = 550
Static Constant ptabpos = 260
Static Constant gwidth = 640

// menu placement
Menu "Packages"
	SubMenu "Panels"
		"Image Tools", /Q, init_StartImageTools()
	end
end

// ***
// UPDATER HOOK FUNCTION

Function AfterCompiledHook()

	if (!DataFolderExists(k_fullPackageFolder))
		init_InitializePackage()
	endif

	DFREF fpf = $k_fullPackageFolder
	NVAR/Z/SDFR=fpf k_imgTVersion
	string astr, estr
	astr = "Updating current undefined version of \rImage Tools to version " + k_version + ".\r\r"
	estr = "The image window and panel will close and reopen.\rSettings may change, but data will not change."
	astr += estr
	variable how = 0
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder fpf
	if (!NVAR_Exists(k_imgTVersion))
		DoAlert/T="Updating Image Tools", 0, astr
		how = 1
	else
		if (k_imgTVersion != k_version)
			astr = "Updating version of Image Tools " + num2str(k_imgTVersion) + " to version " + k_version + ".\r\r"
			astr += estr
			DoAlert/T="Updating Image Tools", 0, astr
			how = 1
		endif
	endif
	SetDataFolder cdf
	if (how != 0)
		DoWindow/K imgT_display
		imgT_Update(k_version)
		init_StartImageTools()
		NVAR/SDFR=fpf k_imgTVersion
		k_imgTVersion = k_version
	endif
	return 1
end

// updates occur within the imgT package folder
Function imgT_Update(svn)
	string svn
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder $k_fullPackageFolder
	
	strswitch(svn)
		case "2.11":
		case "2.10":
			variable/G k_imgTVersion
			make/N=1/O scalewx, scalewy, scaledx
			break
	endswitch
	
	SetDataFolder cdf
	
	return 0
end

// ***
// STRUCTURES

// structure to collect folder + file values
Structure S_iDisplay
	string folder
	string file
EndStructure

// fill the folder + file structure from panel
Function Sf_GetImageDir(iD)
	STRUCT S_iDisplay &iD
	
	ControlInfo/W=$k_fullpanel popupfolder_tabM
	id.folder = "imgT_" + s_value
	ControlInfo/W=$k_fullpanel popupfile_tabM
	id.file = "img_" + s_value
	
	return 0
end

// structure to collect marquee or hRoI coordinates
Structure S_MarqueeCoordinates
	variable hasmarquee
	variable hasdrawing
	variable paleft
	variable paright
	variable patop
	variable pabottom
	variable pgleft
	variable pgright
	variable pgtop
	variable pgbottom
EndStructure

// fill the structure
Function Sf_GetMCCoordinates(mc)
	Struct S_MarqueeCoordinates &mc
	
	DoUpdate/W=$k_imgDisplay
	GetMarquee/W=$k_imgDisplay/Z
	mc.hasmarquee = v_flag
	DrawAction/W=$k_imgDisplay/L=ProgFront getgroup=hRoI, extractOutline
	mc.hasdrawing = v_flag
	
	variable pntsh, pxlsh, pntsppxl

	// hRoI drawing overrides marquee
	if (mc.hasdrawing == 1)
		wave w_polyx, w_polyy
		mc.pgleft=w_polyx[0]
		mc.pgright=w_polyx[1]
		mc.pgtop=w_polyy[1]
		mc.pgbottom=w_polyy[2]
		getwindow $k_imgDisplay, psizeDC
		pxlsh = V_right-V_left
		getwindow $k_imgDisplay, psize
		pntsh = V_right-V_left  
		pntsppxl = pntsh / pxlsh
		mc.paleft = PixelFromAxisVal(k_imgDisplay,"left",mc.pgleft)*pntsppxl
		mc.patop = PixelFromAxisVal(k_imgDisplay,"top",mc.pgtop)*pntsppxl
		mc.paright = PixelFromAxisVal(k_imgDisplay,"left",mc.pgright)*pntsppxl
		mc.pabottom = PixelFromAxisVal(k_imgDisplay,"top",mc.pgbottom)*pntsppxl
		killwaves/Z w_polyx, w_polyy
		return 0
	endif	
	
	// no hRoI drawing or marquee
	if (mc.hasmarquee == 0)
		mc.paleft = NaN; mc.patop = NaN; mc.paright = NaN; mc.pabottom = NaN
		mc.pgleft = NaN; mc.pgtop = NaN; mc.pgright = NaN; mc.pgbottom = NaN
		return 0
	endif
	
	// marquee
	mc.paleft = v_left
	mc.paright = v_right
	mc.patop = v_top
	mc.pabottom = v_bottom	
	GetMarquee/W=$k_imgDisplay/Z left,top
	mc.pgleft = v_left
	mc.pgright = v_right
	mc.pgtop = v_top
	mc.pgbottom = v_bottom	
	return 0
end

// fill the folder + file structure manually
// pass paramters with prefixes!!!
Function SetImagetoShow(sfolder, sfile)
	string sfolder, sfile
	
	// globals for package	
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string flist
	variable itn
	
	// set the folder
	cwdfolder = sfolder	
	flist = f_LoimgT()
	itn = WhichListItem(sfolder[5,inf],flist) + 1	
	Popupmenu popupfolder_tabM, win=$k_fullpanel, mode=itn
	
	// set the file
	cwfile = sfile
	flist = f_LoimgF()
	itn = WhichListItem(sfile[5,inf],flist) + 1
	Popupmenu popupfile_tabM, win=$k_fullpanel, mode=itn
	
	return 0
end

// ***
// HOOK FUNCTIONS
//

// image window hook function
Function Hf_imgTImageHook(iwH)		// WindowHookFunction
	STRUCT WMWinHookStruct &iwH
	
	variable hookResult = 0, ctab
	
	switch(iwH.eventCode)
		case 5:		// mouseup -- check for Marquee
#ifdef DEBUG
			print "mouseup on image"
#endif
			ControlInfo/W=$k_fullpanel tab_main
			ctab = v_value
			disp_UpdatePanelControls(-1)
			disp_UpdatePanelControls(ctab)
			hookResult = 1
			break
	endswitch
	
	return hookResult
end

// histogram window hook function
Function Hf_imgTHistogramsHook(igH)		// WindowHookFunction
	STRUCT WMWinHookStruct &igH
	
	variable hookResult = 0, cx, cy
	string dtrace

	switch(igH.eventCode)
		case 7:		// cursor moved
#ifdef DEBUG
			print "cursor moved on histograms"
#endif
			GetWindow/Z $k_panelName exterior
			if (v_value==1)
#ifdef DEBUG
				print "setting cursor display values on panel: ", xcsr(A,k_fullhgraphs)
#endif
			// first time putting up the cursor
				ValDisplay valdispcx_tabM, win=$k_panelName, value=_NUM:xcsr(A,k_fullhgraphs)
				ValDisplay valdispcy_tabM, win=$k_panelName, value=_NUM:vcsr(A,k_fullhgraphs)
			endif
			hookresult = 1
			break
	endswitch
	
	return hookResult
end

// panel hook function
Function Hf_imgTPanelHook(ipH)		// WindowHookFunction
	STRUCT WMWinHookStruct &ipH
	
	variable hookResult = 0, il, ctab
	
	switch(ipH.eventCode)
		case 0:
			ControlInfo/W=$k_fullpanel tab_main
			ctab = v_value
#ifdef DEBUG
			print "active at tab", ctab
#endif
			disp_UpdatePanelControls(-1)
			disp_UpdatePanelControls(ctab)
			hookResult = 1
			break
	endswitch
	
	return hookResult	
end

// switch tabs
Function disp_SwitchTab(ptab, ctab)
	string ptab
	variable ctab
	
#ifdef DEBUG
	print "Switching from tab to tab ", ptab, ctab
#endif
	
	string tabList, ctabstr=num2str(ctab)
	
	// hide prior controls
	ptab = "*_tab"+ptab
	tabList = ControlNameList(k_panelName,";",ptab)
	ModifyControlList tabList win=$k_fullpanel, disable=1
	
	// show new controls
	ptab = "*_tab"+ctabstr
	tabList = ControlNameList(k_panelName,";",ptab)
	ModifyControlList tabList win=$k_fullpanel, disable=0

	// set current tab value
	TabControl tab_main,win=$k_panelName,userdata(ptab)=ctabstr
	
	// clean up controls in special cases
	disp_UpdatePanelControls(ctab)		
	
	ControlUpdate/A/W=$k_panelName

	return 0
end

// change bit depth
Function ChangeImageBitDepth(minmax,fbd)
	variable minmax, fbd

	// globals for package	
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwfile

	variable bd = (2^f_ImageBitDepth()) - 1
	
	switch(minmax)
		case -2:		// monochrome onoff
			ModifyImage/W=$k_imgDisplay $cwfile, explicit=fbd
			break
		case -1:		// change min
			ModifyImage/W=$k_imgDisplay $cwfile, ctab={fbd*bd,,Grays,0}
			break
		case 0:		// reverse
			ModifyImage/W=$k_imgDisplay $cwfile, ctab={,,,fbd}
			break
		case 1:		// change max
			ModifyImage/W=$k_imgDisplay $cwfile, ctab={,fbd*bd,Grays,0}
			break
	endswitch
	
	return 0

end

// ***
// STARTUP

// starting point from menu call
Function init_StartImageTools()

	string lwfile
	variable first = 0
	
	// initialize?	
	if (!DataFolderExists(k_fullpackageFolder))
		init_InitializePackage()
		first = 1
	endif
	
	imgT_Update(k_version)
	
	// bring window forward or create?
	DoWindow/F $k_imgDisplay
	if (v_flag == 0)
		// validate working folder and file
		DFREF pf = $k_fullpackageFolder
		SVAR/SDFR=pf cwdfolder, cwfile
		if (!first)
			lwfile = "root:" + cwdfolder + ":" + cwfile
			wave swave = $lwfile
			if (!waveExists(swave))
				cwdfolder = StringFromList(0,f_LoimgT())
				cwfile = "img_" + StringFromList(0,f_LoimgF(sfolder=cwdfolder))
			endif
		endif
		// create new windows
		disp_CreateImageWindow()
		disp_AppendHistograms(0)
		disp_AppendImageToolsPanel()
		// initialize the tabs
		if (f_NImagesLoaded() == 0)
			disp_InitializeTabMenus(-1)
		else
			disp_InitializeTabMenus(100)
			disp_ResetImageFileonPanel()
			TM_UpdateContactPrint()
		endif
		disp_SetHookFunctions()
	endif
		
	return 0
end

// set all window hook functions
Function disp_SetHookFunctions()

	SetWindow $k_imgDisplay, hook(imgTImageHook)=Hf_imgTImageHook
	SetWindow $k_histPanel, hook(imgTHistHook)=Hf_imgTHistogramsHook
	SetWindow $k_panelName, hook(imgTPanelHook)=Hf_imgTPanelHook

	return 0
end

// initialize the package folder
Static Function init_InitializePackage()

	// create package folder
	SetDataFolder root:	
	NewDataFolder/O/S Packages	
	NewDataFolder/S $k_packageFolder
	
	// create globals
	// working directory
	string/G cwdfolder = "Packages:imgT", cwfile = "img_default"
	make/N=1/O scalewx, scalewy, scaledx
	make/N=(600,400)/B/U img_default
	img_default = 200*sin(x)*cos(y)+200
	NewDataFolder/O contactprint
	
	SetDataFolder root:

	return 0
end

// create image window
Function disp_CreateImageWindow()

	// globals for package	
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	wave/SDFR=pf/Z scalewx, scalewy, scaledx
	
	string stitle, fstr, tstr
	
	// set the folder and image
	fstr = "root:" + cwdfolder
	DFREF imgF = $fstr	
	wave/SDFR=imgF itd = $cwfile

	variable bd = (2^f_ImageBitDepth()) - 1

	NewImage/N=$k_imgDisplay itd
	MoveWindow/W=$k_imgDisplay 50, 10,-1,-1
	ModifyImage/W=$k_imgDisplay $cwfile, explicit=0
	ModifyGraph/W=$k_imgDisplay height={Plan,1,left,top}
	ModifyImage/W=$k_imgDisplay $cwfile, ctab={0,bd,Grays,0}
	
	// add scale bar (initially in hidden state)
	AppendToGraph/W=$k_imgDisplay/T scalewy vs scalewx
	ErrorBars/L=3 scalewy X,wave=(scaledx,scaledx)
	ModifyGraph/W=imgT_display/Z hidetrace(scalewy)=1, mode(scalewy)=2, rgb(scalewy)=(65535,65534,49151)
	
	disp_AddControlBar()
	
	return 0
end

// add control bar
Function disp_AddControlBar()

	variable nimgs = f_NImagesLoaded()
	
	ControlBar 70
	
	PopupMenu popupSplit_CB, pos={5,2},size={80,20},fSize=12,title="Split"
	PopupMenu popupSplit_CB, mode=1, value=f_SplitOptions(), proc=X_PopMenuProc
	
	SetVariable setvarimgMag_CB pos={5,25},size={70,20},limits={0.2,2,0.1},title="mag"
	SetVariable setvarimgMag_CB fSize=10,format="%2.1f",value=_NUM:1,proc=X_SetVarProc
	
	CheckBox checkShowAxes_CB, pos={90,25},size={20,20},fSize=10,title="Axes",value=1,proc=X_CheckProc
	
	PopupMenu popupScaleBar_CB, pos={5,45},size={80,20},fSize=12,title="Scale Bar"
	PopupMenu popupScaleBar_CB, mode=1, value="None;LT;RT;RB;LB;", proc=X_PopMenuProc

	PopupMenu popupSBColor_CB, pos={130,45}, title="Color",fSize=10, disable=2
	PopupMenu popupSBColor_CB, value="*COLORPOP*",proc=X_PopMenuProc,popColor=(0,0,0)
	
	Slider sliderIMin_CB, pos={160,1},size={80,20},vert=0,limits={0,0.5,0.05},value=0,fSize=10,proc=X_SliderProc

	Button buttonResetIRange_CB, pos={245,10},size={40,22},title="<-->"
	Button buttonResetIRange_CB,fSize=10,proc=X_ButtonProc,disable=0

	Slider sliderIMax_CB, pos={290,1},size={80,20},vert=0,limits={0.5,1,0.05},value=1,fSize=10,proc=X_SliderProc

	CheckBox checkInvert_CB,pos={240,45},size={20,20},title="Invert"
	CheckBox checkInvert_CB,fSize=10,value=0,proc=X_CheckProc

	CheckBox checkBW_CB,pos={300,45},size={20,20},title="B&W"
	CheckBox checkBW_CB,fSize=10,value=0,proc=X_CheckProc
	
	Button buttondoMovie_CB, pos={380,2}, size={30,20}, proc=X_ButtonProc
	Button buttondoMovie_CB, fSize=10, title="🎥", disable=(nimgs<1)
	
	SetVariable setvarsleep_CB, title="sleep",pos={380,25},size={80,20},proc=X_SetVarProc,fSize=10
	SetVariable setvarsleep_CB, limits={0,100,10}, value=_NUM:0, disable=(nimgs<1)

	return 0
end

// show histograms
// how = 0 new, 1 replace traces
Function disp_AppendHistograms(how)
	variable how

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string workf = "root:" + cwdfolder
	DFREF wf = $workf

	variable dhmin, dhmax, xmid
	string imgroot, iHwave, iCHwave, iDHwave, dhistlimits
	string alltraces, htracen, htracel, ctrace, dtrace

	// regenerate the histogram each time
	GenerateHistograms(-1)
	
	workf += ":" + k_histFolder
	DFREF hf = $workf
	
	imgroot = StringFromList(1,cwfile,"_")
	iHwave = "hist_" + imgroot
	iCHwave = "chist_" + imgroot
	iDHwave = "dhist_" + imgroot
	
	wave/SDFR=hf theH = $iHwave
	wave/SDFR=hf theCH = $iCHwave
	wave/SDFR=hf theDH = $iDHwave
	
	WaveStats/Q theDH
	dhmax = v_max
	dhmin = v_min
	sprintf dhistlimits, "%d,%d",dhmin,dhmax
	
	// how = 0 is new; how = 1 is replace
	switch(how)
		case 0:		// new
			DoWindow/F $k_imgDisplay
#ifndef DEBUG
			NewPanel/EXT=2/HOST=$k_imgDisplay/N=$k_histPanel/W=(0,0,gwidth,0.25*gwidth)/K=2
			ModifyPanel noEdit=1, fixedSize=1
#else
			NewPanel/EXT=2/HOST=$k_imgDisplay/N=$k_histPanel/W=(0,0,gwidth,0.25*gwidth)
#endif
			Display/HOST=$k_histPanel/N=$k_histGraphs
			ModifyGraph width = gwidth, height=0.25*gwidth
			AppendtoGraph/R=lhist theH
			htracel = StringFromList(0,TraceNameList("",";",1))
			AppendtoGraph/L=nhist theH
			htracen = StringFromList(1,TraceNameList("",";",1))
			AppendToGraph/L=chist theCH
			ctrace = StringFromList(2,TraceNameList("",";",1))
			AppendtoGraph/R=dhist theDH
			dtrace = StringFromList(3,TraceNameList("",";",1))
			xmid = round(numpnts(theDH)/2)
			
			ModifyGraph rgb=(52428,1,1)
			ModifyGraph lsize=2,standoff=0
			ModifyGraph rgb($htracen)=(1,4,52428), rgb($ctrace)=(1,4,52428)
			ModifyGraph log(lhist)=1,freePos(chist)={0,bottom},freePos(dhist)={inf,bottom}
			ModifyGraph noLabel(lhist)=2,noLabel(lhist)=2,nticks(nhist)=0,nticks(nhist)=0,tick(lhist)=3
			ModifyGraph noLabel(chist)=2,noLabel(dhist)=2,nticks(chist)=0,nticks(dhist)=0,zero(dhist)=2
			ModifyGraph mode($htracen)=1,mode($htracel)=1,lsize($htracen)=1,lsize($htracel)=1
			ModifyGraph freePos(lhist)={inf,bottom},freePos(nhist)={0,bottom}
			ModifyGraph axisEnab(nhist)={0,0.5},axisEnab(lhist)={0,0.5},noLabel(nhist)=2
			ModifyGraph axisEnab(chist)={0.51,1},axisEnab(dhist)={0.51,1}
			ModifyGraph tick(bottom)=1,noLabel(bottom)=2
			Cursor A $dtrace, xmid
			ScaleDHistogram(1)
			break
		case 1:		// just replace waves
			alltraces = SortList(TraceNameList(k_fullhgraphs,";",1))
			// alphabetical sort: c, d, h
			ctrace = StringFromList(0,alltraces)
			ReplaceWave/W=$k_fullhgraphs trace=$ctrace, theCH
			dtrace = StringFromList(1,alltraces)
			ReplaceWave/W=$k_fullhgraphs trace=$dtrace, theDH
			htracel = StringFromList(3,alltraces)
			ReplaceWave/W=$k_fullhgraphs trace=$htracel, theH
			htracen = StringFromList(2,alltraces)
			ReplaceWave/W=$k_fullhgraphs trace=$htracen, theH
			//DoUpdate
			if (strlen(CsrInfo(A,k_fullhgraphs))==0)
				ValDisplay valdispcx_tabM, win=$k_fullpanel, value=_NUM:NaN
				ValDisplay valdispcy_tabM, win=$k_fullpanel, value=_NUM:NaN
			else
				ValDisplay valdispcx_tabM, win=$k_fullpanel, value=_NUM:xcsr(A,k_fullhgraphs)
				ValDisplay valdispcy_tabM, win=$k_fullpanel, value=_NUM:vcsr(A,k_fullhgraphs)
			endif
			// scale the histogram according to slider
			ControlInfo/W=$k_fullpanel slidermagdhist_tabM
			ScaleDHistogram(v_value)
			break
	endswitch
	SetWindow $k_histPanel userdata(dhistogram)=dhistlimits

	return 0
end

// change scale bar color
Function disp_SetScaleBarColor(cstr)
	string cstr

	string tstr, estr = "(.+)(\\K.+\))(.+)", zstr, kstr,fstr
	
	ControlInfo/W=$k_imgDisplay popupSBColor_CB	
	ModifyGraph/W=$k_imgDisplay rgb(scalewy)=(V_red,V_green,V_blue)
	
	tstr = AnnotationInfo(k_imgDisplay,"sctag",0)
	tstr = StringFromList(ItemsInList(tstr)-1,tstr)
	SplitString/E=estr tstr[5,inf], zstr,kstr,fstr
	
	tstr = "\Z24\K" + cstr + fstr
	
	Tag/C/N=sctag tstr
	
	return 0	
end
// set scale bar
Function disp_ShowScaleBar(how)
	variable how
	
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile	
	wave/SDFR=pf scalewx, scalewy, scaledx
	
	variable iscale, idx, idy, ix, iy, ipx, ipy, sp
	string tstr = "\\Z24\\K(65535,65534,49151)"

	if (how == 0)
		Tag/K/N=sctag
		ModifyGraph/W=$k_imgdisplay/Z hidetrace(scalewy)=1
		return 0
	endif
	
	// get image scale settings
	iscale = f_ImageSize()[0]
	ix = f_ImageSize()[1]
	iy = f_ImageSize()[2]
	ipx = f_ImageSize()[3]
	ipy = f_ImageSize()[4]
	sp = floor(log(ipx)) - 1
	idx = 10^sp
	sp = floor(log(ipy)) - 1
	idy = 10^sp
	
	// determine best settings
	switch(iscale)
		case 0:		// no scale (pixels)
			scaledx = idx/2
			sprintf tstr, "%s%d pixels", tstr, idx
			break
		default:		// scale in meters
			sp = floor(log(ix)) - 1
			scaledx = ipx/20
			switch(sp)
				case -9:
					sprintf tstr, "%s%1.0f nm", tstr, 10^(sp+9)
					break
				case -6:
					sprintf tstr, "%s%1.0f um", tstr, 10^(sp+6)
					break
				case -3:
					sprintf tstr, "%s%1.0f mm", tstr, 10^(sp+3)
					break
				case -2:
					sprintf tstr, "%s%1.0f cm", tstr, 10^(sp+2)
					break
				case 0:
					sprintf tstr, "%s%1.0f m", tstr, 10^(sp)
					break
				case 3:
					sprintf tstr, "%s%1.0f km", tstr, 10^(sp-3)
					break
				default:
					sprintf tstr, "%s%3.4f m", tstr, 10^(sp)
					break
			endswitch
			break
	endswitch
	
	switch(how)
		case 1:		// on LT
			scalewx = 2.5*idx
			scalewy = 1.5*idy
			ModifyGraph/W=$k_imgdisplay/Z hidetrace(scalewy)=0
			break
		case 2:		// on RT
			scalewx = ipx - 2.5*idx
			scalewy = 1.5*idy
			ModifyGraph/W=$k_imgdisplay/Z hidetrace(scalewy)=0
			break
		case 3:		// on RB
			scalewx = ipx - 2.5*idx
			scalewy = ipy - 1*idy
			ModifyGraph/W=$k_imgdisplay/Z hidetrace(scalewy)=0
			break
		case 4:		// on LB
			scalewx = 2.5*idx
			scalewy = ipy - 1*idy
			ModifyGraph/W=$k_imgdisplay/Z hidetrace(scalewy)=0
			break
	endswitch
	Tag/C/N=sctag/F=0/A=MB/L=0/B=1/X=0/Y=1 scalewy, 0,tstr

	return 0
end

// turn on/off axes
Function disp_ShowAxes(onoff)
	variable onoff
	
	switch(onoff)
		case 0:		// off
			ModifyGraph nticks(top)=0, nticks(left)=0
			break
		case 1:
			ModifyGraph nticks(top)=12, nticks(left)=12
			break
	endswitch
	return 0
end

// show/hide histogram tab
// how = 0 show
// how = 1 hide
Function disp_ShowHideHistograms(how)
	variable how
	
	SetWindow $k_histPanel, hide=how
	
	return 0
end

// append the panel
Function disp_AppendImageToolsPanel()
	
	// panel frame
	DoWindow/F $k_imgDisplay
	NewPanel/EXT=0/HOST=$k_imgDisplay/W=(0,0,pwidth,pheight)/N=$k_panelName/K=1 as "Image Tools Panel"
#ifndef DEBUG
	ModifyPanel fixedSize=1, noEdit=1
#endif
	
	// version information
	SetDrawEnv fsize= 9
	DrawText pwidth-80,15,"version: " + k_version
	
	// tab controls
	
	TabControl tab_main,pos={0,ptabpos},size={pwidth,pheight-ptabpos},proc=X_TabProc

	TabControl tab_main,fSize=16,tabLabel(0)="Load"
	//TabControl tab_main,help={"Load TIFF images."}

	TabControl tab_main,fSize=16,tabLabel(1)="Calibrate"
	//TabControl tab_main,help={"Calibrate xy,z."}

	TabControl tab_main,fSize=16,tabLabel(2)="Process"
	//TabControl tab_main,help={"Process: Threshold"}

	//TabControl tab_main,fSize=16,tabLabel(3)="Plug-Ins"
	//TabControl tab_main,help={"Plug-Ins"}
	
	disp_Tab2Draw(pwidth,pheight, ptabpos)
	disp_Tab1Draw(pwidth,pheight, ptabpos)
	disp_Tab0Draw(pwidth,pheight,ptabpos)
	disp_TabMDraw()
	
	return 0
end

// draw tabMAIN
Function disp_TabMDraw()

	variable tvoffset = 10
	variable nimgs = 0
	
	// tab MAIN - common to all

	// Display Image Section
	
	TitleBox display_tabM,pos={15,tvoffset},size={122,22},title="Image Display"
	TitleBox display_tabM,fSize=18,frame=0,fStyle=1
	
	ValDisplay valdispNimgs_tabM title="Sets Loaded",pos={180,tvoffset+2},size={120,20}, fSize=12
	ValDisplay valdispNimgs_tabM value=f_NImagesLoaded()

	// current image information
	
	tvoffset += 25
	
	nimgs = f_NImagesLoaded()
	if (nimgs > 1)
		Button buttonleft_tabM title="<",pos={20,tvoffset},size={20,20},proc=X_ButtonProc,fSize=14,disable=2
		Button buttonright_tabM title=">",pos={45,tvoffset},size={20,20},proc=X_ButtonProc,fSize=14,disable=0
	else
		Button buttonleft_tabM title="<",pos={20,tvoffset},size={20,20},proc=X_ButtonProc,fSize=14,disable=2
		Button buttonright_tabM title=">",pos={45,tvoffset},size={20,20},proc=X_ButtonProc,fSize=14,disable=2
	endif

	PopupMenu popupfolder_tabM,pos={65,tvoffset},size={175,23},proc=X_PopMenuProc,title=""
	PopupMenu popupfolder_tabM,fSize=14,mode=1,value= #"f_LoimgT()",disable=0
	
	tvoffset += 25
	
	PopupMenu popupfile_tabM,pos={20,tvoffset},size={106,23},proc=X_PopMenuProc,title=""
	PopupMenu popupfile_tabM,fSize=14,mode=1,value= #"f_LoimgF()",disable=0
	
	nimgs = f_NImagesinStack()
	SetVariable setvarSN_TabM, title="",pos={220,tvoffset+2},size={50,20},proc=X_SetVarProc,fSize=10
	SetVariable setvarSN_TabM, limits={0,nimgs-1,1}, value=_NUM:0, disable=(nimgs<1)

	tvoffset += 20
			
	SetVariable setvarSize_tabM,pos={20,tvoffset+2},title="XY Size",size={280,20},fSize=10
	SetVariable setvarSize_tabM,value=_STR:f_ImageSizeText(),noedit=1
	ValDisplay valdispBitDepth_tabM,pos={320,tvoffset+2},title="Bit Depth",size={80,20},fSize=10
	ValDisplay valdispBitDepth_tabM,value=_NUM:f_ImageBitDepth()
	
	tvoffset += 22
	
	SetVariable setvarImgNote_tabM,pos={20,tvoffset},title="Notes",size={380,20},fSize=10
	SetVariable setvarImgNote_tabM,value=_STR:f_ImageNoteText(), proc=X_SetVarProc

	tvoffset += 25
	
	CheckBox checkCP_tabM,pos={20,tvoffset},size={20,20},title="use in contact print"
	CheckBox checkCP_tabM,fSize=10,value=0,proc=X_CheckProc,disable=0
	
	ValDisplay valdispccp_tabM, pos={160,tvoffset+1}, title="contact prints"
	ValDisplay valdispccp_tabM size={100,20},value=_NUM:0,fSize=10
	
	Button buttonClearAll_tabM, pos={pwidth - 130,tvoffset-10},size={120,22},title="Clear Contact Images"
	Button buttonClearAll_tabM,fSize=10,proc=X_ButtonProc,disable=0
	
	Button buttonDoCPrint_tabM, pos={pwidth - 130,tvoffset+10},size={120,22},title="Make Contact Layout"
	Button buttonDoCPrint_tabM,fSize=10,proc=X_ButtonProc,disable=0
	
	tvoffset += 30

	SetVariable setvarCPNote_tabM,pos={20,tvoffset},title="Notes",size={380,20},fSize=10
	SetVariable setvarCPNote_tabM,value=_STR:""

	// Histogram Control Section
	
	tvoffset += 25

	CheckBox checkshowHist_tabM,pos={15,tvoffset},size={20,20},title="Histogram Display"
	CheckBox checkshowHist_tabM,fSize=16,fStyle=1,value=1,proc=X_CheckProc,disable=0
		
	Button buttonRedoH_tabM,pos={pwidth-50,tvoffset},size={40,25},proc=X_ButtonProc,title="Redo"
	Button buttonRedoH_tabM,fSize=10,disable=0
	
	tvoffset += 25
	
	//TitleBox hset_tabM,pos={pwidth-80,tvoffset},title="cursor",fSize=10,frame=0,disable=0
	
	ValDisplay valdispcx_tabM pos={pwidth-65,tvoffset+5}, title="x",size={60,20},value=_NUM:xcsr(A,k_fullhgraphs),fSize=10
	ValDisplay valdispcy_tabM pos={pwidth-85,tvoffset+25}, title="y",size={80,20},value=_NUM:vcsr(A,k_fullhgraphs),fSize=10
	
	//tvoffset += 25

	CheckBox checksethRoI_tabM,pos={15,tvoffset},size={20,20},title="restrict\rto RoI"
	CheckBox checksethRoI_tabM,fSize=10,value=0,proc=X_CheckProc,disable=0,userdata(state)="0"

	CheckBox checklockhRoI_tabM,pos={70,tvoffset},size={20,20},title="lock\rRoI"
	CheckBox checklockhRoI_tabM,fSize=10,value=0,proc=X_CheckProc,disable=0,userdata(state)="0"

	SetVariable mqx0_tabM, title="<",pos={115,tvoffset-5},size={80,20},value=_NUM:0,disable=0,proc=X_SetVarProc
	SetVariable mqy0_tabM, title="^",pos={205,tvoffset-5},size={80,20},value=_NUM:0,disable=0,proc=X_SetVarProc
	SetVariable mqdx_tabM, title="-",pos={115,tvoffset+15},size={80,20},value=_NUM:0,disable=0,proc=X_SetVarProc
	SetVariable mqdy_tabM, title="|",pos={205,tvoffset+15},size={80,20},value=_NUM:0,disable=0,proc=X_SetVarProc
	
	TitleBox histscale_tabM,pos={295,tvoffset-25},size={122,22},title="dI/dB"
	TitleBox histscale_tabM,fSize=10,frame=0,disable=0
	
	Slider slidermagdhist_tabM pos={295,tvoffset-10},value=0,limits={0,3,0.2}
	Slider slidermagdhist_tabM proc=X_SliderProc,size={60,50},ticks=0,disable=0

	return 0
end

// ***
// SUPPORT FUNCTIONS

// initialize menu options for a given tab
Function disp_InitializeTabMenus(tab)
	variable tab
	
	DFREF pf=$k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
#ifdef DEBUG
	print "initializing tab ", tab
#endif

	switch(tab)
		case -1:		// initialize package
			CheckBox/Z checkCP_tabM, win=$k_fullpanel, value=0, disable=2
			CheckBox/Z checksethRoI_tabM, win=$k_fullpanel, value=0, disable=2,userdata(state)="2"
			CheckBox/Z checklockhRoI_tabM, win=$k_fullpanel, value=0, disable=2,userdata(state)="2"
			ValDisplay valdispccp_tabM win=$k_fullpanel, value=_NUM:f_CountofContactPrints(), disable = 2
			PopupMenu/Z popupfolder_tabM win=$k_fullpanel, disable = 2
			PopupMenu/Z popupfile_tabM win=$k_fullpanel, disable = 2
			Button/Z buttonleft_tabM win=$k_fullpanel, disable = 2
			Button/Z buttonright_tabM win=$k_fullpanel, disable = 2
			Button/Z buttonClearAll_tabM win=$k_fullpanel, disable = 2
			Button/Z buttonDoCPrint_tabM win=$k_fullpanel, disable = 2
			break
		case 1:
			break
		case 100:		// load first image
			CheckBox/Z checkCP_tabM, win=$k_fullpanel, value=0, disable=0
			CheckBox/Z checksethRoI_tabM, win=$k_fullpanel, value=0, disable=0
			ValDisplay valdispccp_tabM win=$k_fullpanel, value=_NUM:f_CountofContactPrints(), disable = 0
			PopupMenu/Z popupfolder_tabM win=$k_fullpanel, disable = 0
			PopupMenu/Z popupfile_tabM win=$k_fullpanel, disable = 0
			Button/Z buttonClearAll_tabM win=$k_fullpanel, disable = 2
			Button/Z buttonDoCPrint_tabM win=$k_fullpanel, disable = 2
			SetImagetoShow(cwdfolder, cwfile)
			disp_ResetImageFileonPanel()
			break
		default:
//			CheckBox/Z checklockhRoI_tabM, win=$k_fullpanel, value=0, disable=2
//			ValDisplay valdispccp_tabM win=$k_fullpanel, value=_NUM:f_CountofContactPrints()
//			// no folder or image set for tab 1 menus
//			if (strlen(cwdfolder)==0)
//				// set to first folder and first file
//				cwdfolder = "imgT_" + StringFromList(0,f_LoimgT())
//				cwfile = StringFromList(0,f_LoimgF(sfolder=cwdfolder))
//			endif
//			SetImagetoShow(cwdfolder,cwfile)
//			ResetImageFileonPanel()
			break
	endswitch
	
	return 0
end

// lock the histogram to a RoI
Function LockHistogramRoI(yn)
	variable yn

	variable ppx0, ppx1, ppy0, ppy1
	STRUCT S_MarqueeCoordinates mc
	Sf_GetMCCoordinates(mc)

	switch(yn)
		case 0:		// unlock
			SetDrawLayer/W=$k_imgDisplay ProgFront
			DrawAction/W=$k_imgDisplay getgroup=hRoI, delete
			SetDrawLayer/W=$k_imgDisplay UserFront
			SetMarquee/W=$k_imgDisplay, mc.paleft,mc.patop,mc.paright,mc.pabottom
			CheckBox/Z checksethRoI_tabM, win=$k_fullpanel, disable=0
			break
		case 1:		// lock
			// get marquee from image graph	
			if (!mc.hasmarquee && !mc.hasdrawing)
				//if (!quiet)
				//	DoAlert/T="Restrict Histogram" 0, "No marquee region or locked histogram drawing is present. Set one a marquee try again."
				//endif
				Checkbox checklockhRoI_tabM, win=$k_fullpanel, value=0
				return 0
			endif
			
			// put trunated pixel coordinates in local variables
			ppx0 = trunc(mc.pgleft)
			ppy0 = trunc(mc.pgtop)
			ppx1 = trunc(mc.pgright)
			ppy1 = trunc(mc.pgbottom)
			SetDrawLayer/W=$k_imgDisplay ProgFront
			SetDrawEnv/W=$k_imgDisplay gstart,gname=hRoI
			SetDrawEnv/W=$k_imgDisplay linefgc=(65535,0,0),linethick=2,fillpat=0, xcoord=top,ycoord=left, save
			DrawRect/W=$k_imgDisplay ppx0,ppy0,ppx1,ppy1
			SetDrawEnv/W=$k_imgDisplay gstop
			SetDrawLayer/W=$k_imgDisplay UserFront
			CheckBox/Z checksethRoI_tabM, win=$k_fullpanel, disable=2
			break
	endswitch
	
	return 0
end

// split display windows
Function disp_SplitWindow(how)
	variable how
	
	DFREF pf=$k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string rfile = f_RootFileName()
	string tfile = rfile + "_th"
	string bfile = rfile + "_bg"
	string wdf = "root:" + cwdfolder
	
	switch(how)
		case 0:		// refresh entirely (with no prior information)
			rfile = ImageNameList(k_imgDisplay,";")
			if (ItemsInList(rfile) > 1)
				tfile = StringFromList(1,rfile)
				RemoveImage/W=$k_imgDisplay/Z $tfile
				ModifyGraph/W=$k_imgDisplay axisEnab(top)={0,1}
				bfile = StringFromList(0,f_SplitOptions())
				PopupMenu popupSplit_CB, win=$k_imgDisplay, mode=1, value=f_SplitOptions(), userdata=bfile
			endif
			break
		case 1:		// none
			RemoveImage/W=$k_imgDisplay/Z $tfile
			RemoveImage/W=$k_imgDisplay/Z $bfile
			ModifyGraph/W=$k_imgDisplay axisEnab(top)={0,1}
			break
		case 2:		// threshold
			wave/SDFR=$wdf timg = $tfile
			RemoveImage/W=$k_imgDisplay/Z $bfile
			AppendImage/W=$k_imgDisplay/T=splitT timg
			if (f_isStack())
				ModifyImage/W=$k_imgDisplay $tfile, plane=f_PlaneNumber()
			endif
			ModifyGraph/W=$k_imgDisplay axisEnab(top)={0,0.5},axisEnab(splitT)={0.5,1}
			ModifyGraph/W=$k_imgDisplay noLabel(splitT)=2,nticks(splitT)=0,freePos(splitT)=0
			break
		case 3:		// background
			wave/SDFR=$wdf bimg = $bfile
			RemoveImage/W=$k_imgDisplay/Z $tfile
			AppendImage/W=$k_imgDisplay/T=splitT bimg
			if (f_isStack())
				ModifyImage/W=$k_imgDisplay $bfile, plane=f_PlaneNumber()
			endif
			ModifyGraph/W=$k_imgDisplay axisEnab(top)={0,0.5},axisEnab(splitT)={0.5,1}
			ModifyGraph/W=$k_imgDisplay noLabel(splitT)=2,nticks(splitT)=0,freePos(splitT)=0
			break	
	endswitch
	
	return 0
end

// update panel control displays and values for a given tab
Function disp_UpdatePanelControls(tab)
	variable tab
	
	DFREF pf=$k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
#ifdef DEBUG
	print "updating tab ", tab
#endif

	variable iwd, iwf, istack, thlevel
	string lowfolders = f_LoimgT()
	string lowfiles = f_LoimgF(sfolder=cwdfolder)
	string thlevels = ""

	switch(tab)
		case -1:		// main (always present) content
			iwd = WhichListItem(cwdfolder[5,inf],lowfolders)+1
			iwf = WhichListItem(cwfile[4,inf],lowfiles)+1
			lowfiles = "\"" + lowfiles + "\""
			PopupMenu popupfolder_tabM, win=$k_fullpanel, value=f_LoimgT(), mode=iwd, userData=cwdfolder
			PopupMenu popupfile_tabM, win=$k_fullpanel, value=#lowfiles, mode=iwf, userData=cwfile
			// handle case for too few number of images loaded
			if (ItemsInList(f_LoimgT())<2)
				Button/Z buttonleft_tabM, win=$k_fullpanel, disable=2
				Button/Z buttonright_tabM,win=$k_fullpanel, disable=2
			elseif (iwd == 1)
				Button/Z buttonleft_tabM, win=$k_fullpanel, disable=2
				Button/Z buttonright_tabM,win=$k_fullpanel, disable=0
			elseif (iwd == ItemsInList(f_LoimgT()))
				Button/Z buttonleft_tabM, win=$k_fullpanel, disable=0
				Button/Z buttonright_tabM,win=$k_fullpanel, disable=2
			else
				Button/Z buttonleft_tabM, win=$k_fullpanel, disable=0
				Button/Z buttonright_tabM,win=$k_fullpanel, disable=0
			endif
			// handle miscellaneous control settings
			SetVariable setvarSize_tabM, win=$k_fullpanel, value=_STR:f_ImageSizeText()
			SetVariable setvarImgNote_tabM, win=$k_fullpanel, value=_STR:f_ImageNoteText()
			ValDisplay valdispBitDepth_tabM, win=$k_fullpanel, value=_NUM:f_ImageBitDepth()
			istack = f_NImagesinStack()
			SetVariable setvarSN_TabM, win=$k_fullPanel, limits={0,iStack-1,1}, disable=(iStack==0)
			Button buttondoMovie_CB, win=$k_imgDisplay, disable=(iStack==0)
			SetVariable setvarsleep_CB, win=$k_imgDisplay, disable=(iStack==0)
			// show/hide histograms
			ControlInfo/W=$k_fullPanel checkshowhist_tabM
			Checkbox checksethRoI_tabM, win=$k_fullPanel, disable=(2*(v_value==0))
			Checkbox checklockhRoI_tabM, win=$k_fullPanel, disable=(2*(v_value==0))
			Button buttonRedoH_tabM, win=$k_fullPanel, disable=(2*(v_value==0))
			Slider slidermagdhist_tabM, win=$k_fullPanel, disable=(2*(v_value==0))			
			// handle case for marquee present or not
			GetMarquee/Z/W=$k_imgDisplay
			if (v_flag==1)
				SetVariable mqx0_tabM, win=$k_fullpanel, disable=0, value=_NUM:v_left
				SetVariable mqy0_tabM, win=$k_fullpanel, disable=0, value=_NUM:v_top
				SetVariable mqdx_tabM, win=$k_fullpanel, disable=0, value=_NUM:(v_right - v_left)
				SetVariable mqdy_tabM, win=$k_fullpanel, disable=0, value=_NUM:(v_bottom - v_top)
			else
				ValDisplay valdispcx_tabM, win=$k_fullPanel, disable=((v_value==0))
				ValDisplay valdispcy_tabM, win=$k_fullPanel, disable=((v_value==0))
				SetVariable mqx0_tabM, win=$k_fullpanel, disable=2
				SetVariable mqy0_tabM, win=$k_fullpanel, disable=2
				SetVariable mqdx_tabM, win=$k_fullpanel, disable=2
				SetVariable mqdy_tabM, win=$k_fullpanel, disable=2
				Checkbox checksethRoI_tabM, win=$k_fullPanel, disable=2
				Checkbox checklockhRoI_tabM, win=$k_fullPanel, disable=2
			endif
			break
		case 0:		// load tab
			ControlInfo/W=$k_fullpanel popupload_tab0
			istack = v_value == 4 ? 0 : 2
			SetVariable/Z startimg_tab0, win=$k_fullpanel, disable=istack
			SetVariable/Z stepimg_tab0, win=$k_fullpanel, disable=istack
			SetVariable/Z endimg_tab0, win=$k_fullpanel, disable=istack
			SetVariable/Z startind_tab0, win=$k_fullpanel, disable=istack
			SetVariable/Z stepind_tab0, win=$k_fullpanel, disable=istack
			if (v_value == 1)
				CheckBox/Z checkasstack_tab0, win=$k_fullpanel, disable=2, value=0
			else
				CheckBox/Z checkasstack_tab0, win=$k_fullpanel, disable=0
			endif
			break
		case 1:		// calibrate tab
			ControlInfo/W=$k_fullpanel popupmethodxy_tab1
			SetVariable setvarSize_tabM, win=$k_fullpanel, value=_STR:f_ImageSizeText()
			Button buttonclearlocalscale_tab1, win=$k_fullpanel, disable=(f_HasLocalScale()==0)			
			STRUCT S_ProcessVariables spv
			Sf_GetProcessVariables(spv)
			switch(spv.xyhow)
				case 1:	// pixel width
					PopupMenu popupmethodxy_tab1, win=$k_fullpanel, mode=1
					SetVariable psizem_tab1, win=$k_fullpanel, value=_NUM:spv.xympixel
					SetVariable psizelm_tab1, win=$k_fullpanel, value=_NUM:spv.xylmpixel
					break
				case 2:	// image width
					PopupMenu popupmethodxy_tab1, win=$k_fullpanel, mode=2
					SetVariable psizem_tab1, win=$k_fullpanel, value=_NUM:spv.xyiwidth
					SetVariable psizelm_tab1, win=$k_fullpanel, value=_NUM:spv.xyliwidth
					break
			endswitch
//			istack = f_NImagesinStack()
//			Checkbox checkbgtoall_tab1, win=$k_fullPanel, disable=(iStack==0)
			break
		case 2:		// process tab
			ControlInfo/W=$k_fullpanel popupThresMethod_tab2
			switch(v_value)
				case 1:	// manual
					thlevel = xcsr(A,k_fullhgraphs)
					break
				default:	// all others
					if (f_isThreshold())
						thlevels = "root:" + cwdfolder + ":" + cwfile[4,inf] + "levels"
					else
						thlevels = "root:" + cwdfolder + ":" + cwfile[4,inf] + "_thlevels"
					endif
					wave/Z thl = $thlevels
					if (waveexists(thl))
						if (f_isStack())
							thlevel = thl[f_PlaneNumber()]
						else
							thlevel = thl[0]
						endif
					else
						thlevel = xcsr(A,k_fullhgraphs)
					endif
					thlevels = TraceNameList(k_fullhgraphs,";",1)
					thlevels = StringFromList(1,SortList(thlevels))
					Cursor/W=$k_fullhgraphs/S=0 A, $thlevels, thlevel
					break
			endswitch
			SetVariable setvarThresLevel_tab2, win=$k_fullPanel, disable=(2*(v_value!=1)), value=_NUM:thlevel
			CheckBox checkOTPlane_tab2, win=$k_fullPanel, disable=(2*(f_NImagesinStack()==0))
			//DoUpdate
#ifdef DEBUG
	print "reset cursor due to threshold change ", thlevels, thlevel
#endif
			break
	endswitch
	
	return 0
end

// ***
// Contact Prints
end

// generate contact print layout
Function TM_CreateContactPrintLayout()

	// global values
	DFREF cpf = $k_fullpackageFolder + ":contactprint"
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	variable cntp, ic, ip, pc, wsize
	string strName,fulllist="",porl,tilestr,eistr
	
	cntp = f_CountofContactPrints()
	
	if (cntp<=0)
		DoAlert/T="Contact Print", 0, "No images stored as contact prints."
		return 0
	endif

	if (cntp < 3)
		wsize = 8*72/cntp
		porl = "LANDSCAPE"
		tilestr = "Tile/A=(0,2)"
	elseif (cntp < 5)
		wsize = 1.5*8*72/cntp
		porl = "PORTRAIT"
		tilestr = "Tile/A=(0,2)"
	elseif (cntp < 9)
		wsize = 1.5*10.5*72/cntp
		porl = "LANDSCAPE"
		tilestr = "Tile/A=(2,0)"
	elseif (cntp < 12)
		wsize = 1.5*10.5*72/cntp
		porl = "LANDSCAPE"
		tilestr = "Tile/A=(3,0)"
	else
		wsize = 10.5*72/cntp
		porl = "LANDSCAPE"
		tilestr = "Tile/A=(3,0)"
	endif
	
	for (ic=0;ic<cntp;ic+=1)
		strName = GetIndexedObjNameDFR(cpf,3,ic)
		fulllist += strName + ";"
		DoWindow/K/Z $strName
		SVAR/SDFR=cpf exCmd = $strName
		Execute/Q/Z exCmd
		DoWindow/C $strName
		ModifyGraph/W=$strname width=wsize
		DoWindow/HIDE=1 $strName
	endfor
	
	fullList = SortList(fullList)
	
	DoWindow/K/Z imgT_contactlayout
	NewLayout/P=$porl/N=imgT_contactlayout
	for (ic=0;ic<cntp;ic+=1)
		strName = StringFromList(ic,fulllist)
		AppendLayoutObject/F=0 graph $strName
	endfor
	Execute/Q/Z tilestr
	
	// add info to display
	eistr = f_DateTimeStamp() + " " + IgorInfo(1) + " " + IgorInfo(7)
	ControlInfo/W=$k_fullpanel setvarCPNote_tabM
	if (strlen(s_value)!=0)
		eistr += "\t" + s_value
	endif
	TextBox/C/N=ExpInfo/F=0/A=RB/X=0/Y=0 eistr
	return 0
end


// remove a contact print recreaction string
Function TM_ClearAllContactPrintImages()

	// global values
	if (!DataFolderExists(k_fullpackageFolder + ":contactprint"))
		return 0
	endif
	
	DFREF cpf = $k_fullpackageFolder + ":contactprint"
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	variable ic, npts
	string wStr
	
	npts = CountObjectsDFR(cpf,3)
	
	for (ic=0;ic<npts;ic+=1)
		wStr = GetIndexedObjNameDFR(cpf,3,ic)
		KillWindow/Z $wStr
	endfor
	
	KillDataFolder/Z cpf
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder pf
	NewDataFolder/O contactprint
	SetDataFolder cdf

	Checkbox checkCP_tabM, win=$k_fullpanel, value=0
	ValDisplay valdispccp_tabM, win=$k_fullpanel, value = 0
	
	Button buttonDoCPrint_tabM, win=$k_fullpanel, disable=2
	Button buttonClearAll_tabM, win=$k_fullpanel, disable=2
	
	SetVariable setvarCPNote_tabM, win=$k_fullpanel, disable=2
	return 0
end

// remove a contact print recreaction string
Function TM_RemoveContactPrint()

	// global values
	DFREF cpf = $k_fullpackageFolder + ":contactprint"
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string cpstr
	cpstr = cwdfolder[5,inf] + "_" + cwfile

	DFREF cdf = GetDataFolderDFR()
	SetDataFolder cpf
	killstrings/Z $cpstr
	killwindow/Z $cpstr
	
	SetDataFolder cdf

	return 0
end

// store as contact print recreaction
Function TM_StoreasContactPrint([sfolder,sfile])
	string sfolder, sfile

	// global values
	DFREF cpf = $k_fullpackageFolder + ":contactprint"
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	DFREF cdf = GetDataFolderDFR()
	
	string eistr, wname
	
	if (ParamIsDefault(sfolder))
		sfolder = cwdfolder[5,inf]
	endif
	if (ParamIsDefault(sfile))
		sfile = cwfile
	endif

	wname = "root:" + cwdfolder + ":" + cwfile
	
	string cpstrng, cprec, scprec
	string rgex = "((?s)(.)+)\\tNewPanel/HOST=#"
	string pstr, sstr
	
	SetDataFolder pf
	NewDataFolder/O/S contactprint
	
	cpstrng = sfolder + "_" + sfile
	if (f_isStack())
		cpstrng += "_" + num2str(f_PlaneNumber())
	endif
	
	cprec = WinRecreation(k_imgDisplay,0)
	eistr = note($wname)

	SplitString/E=(rgex) cprec, scprec
	
	sstr = "\tModifyGraph expand=0.5,lsize=1"
	sstr += "\r\tTextBox/C/N=name/F=0/A=LT \"\\Z08" + cpstrng + "\""
	if (strlen(eistr)!=0)
		sstr += "\rTextBox/C/N=ImgInfo/F=0/A=RB/X=0/Y=0 " + "\"\\Z08" + eistr +  "\""
	endif
	sstr += "\rEndMacro"
	scprec += sstr
	
	string/G $cpstrng = scprec
	
	SetDataFolder cdf
	
	return 0
end

// update current image as contact print
Function TM_UpdateContactPrint()

	if (f_IsContactPrint()==0)
		Checkbox checkCP_tabM, win=$k_fullpanel, value=0
	else
		TM_StoreasContactPrint()
		Checkbox checkCP_tabM, win=$k_fullpanel, value=1
	endif
	
	ValDisplay valdispccp_tabM win=$k_fullpanel, value=_NUM:f_CountofContactPrints()
	
	if (f_CountofContactPrints() == 0)
		Button buttonDoCPrint_tabM, win=$k_fullpanel, disable=2
		Button buttonClearAll_tabM, win=$k_fullpanel, disable=2
		SetVariable setvarCPNote_tabM, win=$k_fullpanel, disable=2
	else
		Button buttonDoCPrint_tabM, win=$k_fullpanel, disable=0
		Button buttonClearAll_tabM, win=$k_fullpanel, disable=0
		SetVariable setvarCPNote_tabM, win=$k_fullpanel, disable=0
	endif
	
	return 0

end

// ***
// LOAD IMAGES

// create a folder to store an image
Function/S CreateImageFolder(fnsuffix)
	string fnsuffix
	
	SetDataFolder root:
	
	// create the new data folder and subfolder
	string fname = "imgT_" + cleanupname(fnsuffix,0)
	variable ic = 0, etrue = 1
	do
		if (DataFolderExists(fname))
			fname = fname + num2str(ic)
		else
			etrue = 0
		endif
	while(etrue)
	NewDataFolder/S $fname
	
	SetDataFolder root:
	
	return fname
end

// ***
// SHOW IMAGE

// show an image
Function disp_ShowAnImage([plane,reset])
	variable plane,reset

	// globals for package	
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string fstr, tstr
	variable bd
	
	// just moving to new plane
	if (!ParamIsDefault(plane) && (ParamIsDefault(reset)))
		ModifyImage/W=$k_imgDisplay $cwfile, plane=plane
		fstr = f_SplitWindowImage()
		if (strlen(fstr)!=0)
				ModifyImage/W=$k_imgDisplay $fstr, plane=plane
		endif
		return 0
	endif

#ifdef DEBUG
	print "Showing an image ", cwfile
#endif
	
	// set the folder and new image to display
	fstr = "root:" + cwdfolder
	DFREF imgF = $fstr	
	wave/SDFR=imgF itd=$cwfile
	bd = f_ImageBitDepth()
	
	tstr = StringFromList(0,(ImageNameList(k_imgDisplay,";")))
	ReplaceWave/W=$k_imgDisplay image=$tstr, itd
	fstr += ":" + cwfile
	if (f_isStack())
		ModifyImage/W=$k_imgDisplay $cwfile, plane=f_PlaneNumber()
	endif
	return 0
end

// step thru images
// -1 back; 0 stay; 1 forward
Function TM_StepThruImages(how,[plane])
	variable how, plane
	
	string simg = f_LoimgT()
	variable nimg = ItemsInList(simg), itd, nplane

#ifdef DEBUG
	print "Stepping through images ", how
#endif
	
	ControlInfo/W=$k_fullpanel popupFolder_tabM
	itd = v_value

	if (ParamIsDefault(plane))
		switch(how)
			case -1:		// step back one image
				if (itd == 1)
					return 0
				endif
				ResetImageFolder(itd-1)
				break
			case 0:		// new image
				ResetImageFolder(itd)
				break
			case 1:		// step forward one image
				if (itd == nimg)
					return 0
				endif
				ResetImageFolder(itd+1)
				break
		endswitch
		// have we switched to an image stack?
		nimg = f_NImagesInStack()
		if (nimg > 0)
			ControlInfo/W=$k_fullpanel setvarSN_TabM
			if (v_value > nimg-1)
				SetVariable setvarSN_TabM, win=$k_fullpanel, value=_NUM:(nimg-1)
				ControlUpdate/W=$k_fullpanel setvarSN_TabM
				disp_ShowAnImage(plane=(nimg-1),reset=1)
			else
				disp_ShowAnImage(plane=v_value,reset=1)
			endif
		else
			disp_ShowAnImage()
		endif
	else
		disp_ShowAnImage(plane=plane)
	endif
	
	// show histogram only when window is present
	GetWindow $k_fullhPanel hide
	if (v_value==0)
		disp_AppendHistograms(1)
	endif

#ifdef DEBUG
	print "Done stepping through images ", how
#endif
	
	return 0
end

// play movie for stack
Function TM_PlayStack(slt)
	variable slt

	variable ic, ni, cplane

	if (!f_isStack())
		return 0
	endif
	
	ni = f_NImagesinStack()
	cplane = f_PlaneNumber()

#ifdef DEBUG
	print "playing a movie: ", slt
#endif
//	DoWindow/F $k_imgDisplay
//	DoUpdate
	
	try
		if (slt == 0)
			for (ic=0;ic<ni;ic+=1)
				disp_ShowAnImage(plane=ic)
				DoUpdate/W=$k_imgDisplay
				SetVariable setvarSN_TabM, win=$k_fullPanel, value=_NUM:ic
			endfor
		else
			for (ic=0;ic<ni;ic+=1)
				disp_ShowAnImage(plane=ic)
				DoUpdate/W=$k_imgDisplay
				SetVariable setvarSN_TabM, win=$k_fullPanel, value=_NUM:ic
				Sleep/T/Q slt
			endfor
		endif
		disp_ShowAnImage(plane=cplane)
		DoUpdate/W=$k_imgDisplay
		SetVariable setvarSN_TabM, win=$k_fullPanel, value=_NUM:cplane
	catch
		SetVariable setvarSN_TabM, win=$k_fullPanel, value=_NUM:(ic-1)
		TM_StepThruImages(0,plane=(ic-1))
	endtry
	
	DoWindow/F $k_panelName
	return 0
end

// reset marquee coordinates
Function ResetMQCoordinates()

	variable left, top, right, bottom
	
	ControlInfo/W=$k_panelName mqx0_tabM
	left = v_value
	ControlInfo/W=$k_panelName mqy0_tabM
	top = v_value
	ControlInfo/W=$k_panelName mqdx_tabM
	right = left + v_value
	ControlInfo/W=$k_panelName mqdy_tabM
	bottom = top + v_value
	
	SetMarquee/W=$k_imgDisplay left, top, right, bottom
	
	return 0
end

// reset image folder (increment value)
Function ResetImageFolder(fnum)
	variable fnum
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	// reset the global folder
	string flist = f_LoimgT()
	
	flist = f_LoimgT()
	cwdfolder = "imgT_" + StringFromList(fnum-1,flist,";")
	
	// reset the file to the first one in the folder when needed
	flist = f_LoimgF(sfolder=cwdfolder)
	fnum = WhichListItem(cwfile[4,inf],flist)
	if (fnum < 0)
		cwfile = "img_" + StringFromList(0,f_LoimgF(sfolder=cwdfolder),";")
	else
		cwfile = "img_" + StringFromList(fnum,flist)
	endif
	
#ifdef DEBUG
	print "reset image folder/file ", cwdfolder, cwfile
#endif

	return 0
end

Function ResetImageFile(fname)
	string fname
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwfile

	cwfile = "img_" + fname
	return 0
end	
	

// reset image file on panel
Function disp_ResetImageFileonPanel()
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	variable fnum
	string flist = f_LoimgF(), ffile

	// reset the file
	ffile = cwfile[4,inf]
	fnum = WhichListItem(ffile,flist)
	PopupMenu popupfile_tabM, win=$k_fullpanel, mode=fnum+1, popvalue=ffile
	SetVariable setvarImgNote_tabM, win=$k_fullpanel, value=_STR:f_ImageNoteText()
	SetVariable setvarSize_tabM,win=$k_fullpanel, value=_STR:f_ImageSizeText()
	return 0
end

// ***
// PROCESSING

// ***
// HISTOGRAMS

// generate the histogram
// toRoI - 1 means generate them within a defined RoI
Function GenerateHistograms(toRoI)
	variable toRoI
	
	DFREF cdf = GetDataFolderDFR()

	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string dfstr, imgHstr, imgroot, maskWave, sfile
	variable quiet=0, maskordraw, wtype
	variable bflag=0, tflag=0, dhmin, dhmax
	variable isstack=0, stnum, cplane
	
	dfstr = "root:" + cwdfolder
	DFREF wf = $dfstr
	
	imgroot = StringFromList(1,cwfile,"_")
	
	SetDataFolder wf
	
	// do not re-generate histogram for threshold images
	
	if (f_isThreshold())
		sfile = ReplaceString("_th",cwfile,"")
	else
		sfile = cwfile
	endif

	wave wimg = $sfile
	
	// extract only the current stack plane
	if (f_isStack())
		duplicate/FREE wimg sstack_tmp
		redimension/N=(-1,-1) wimg
		cplane = f_PlaneNumber()
		MatrixOp/O wimg = sstack_tmp[][][cplane]
		isstack = 1
	endif

#ifdef DEBUG
	print "generating histogram for ", cwdfolder, cwfile, cplane
#endif
	
	NewDataFolder/O/S $k_histFolder
	
	// 8bit or 16bit	
	wtype = WaveType(wimg,0)	
	if (wtype & 0x08)
		bflag = 0		// 8 bit
	elseif (wtype & 0x10)
		bflag = 1		// 16 bit
	endif
	
	// single channel or three channel
	wtype = WaveDims(wimg)
	if (wtype == 3)
		tflag = 1
	endif
	
	// negative input means read from panel	
	if (toRoI < 0)
		ControlInfo/W=$k_fullpanel checksethRoI_tabM
		toRoI = v_value
		quiet = 1
	endif
	
	// generate histogram	
	switch(toRoI)
		case 0:		// histogram over all image
			if (bflag)
				ImageHistogram/I wimg
			else
				ImageHistogram wimg
			endif
			CheckBox/Z checklockhRoI_tabM, win=$k_fullpanel, value=0, disable=2
			break
		case 1:		// histogram over RoI or not
			if (GenerateHistogramRoIMask(wimg,quiet)!=0)
				maskWave = sfile + "_HRoIMask"
				wave RoIWave=$maskWave
				if (bflag)
					ImageHistogram/I/R=RoIWave wimg
				else
					ImageHistogram/R=RoIWave wimg
				endif
				CheckBox/Z checklockhRoI_tabM, win=$k_fullpanel, disable=0
			else
				if (bflag)
					ImageHistogram/I wimg
				else
					ImageHistogram wimg
				endif
				CheckBox/Z checksethRoI_tabM, win=$k_fullpanel, value=0
				CheckBox/Z checklockhRoI_tabM, win=$k_fullpanel, value=0, disable=2
			endif
			break
	endswitch
	
	switch(tflag)
		case 0:		// single channel image
			wave iHo = W_imageHist
			break
		case 1:		// rgb image
			wave iHRo = W_imageHistR
			wave iHGo = W_imageHistG
			wave iHBo = W_imageHistB
			duplicate/O/FREE iHRo, iHo
			iHo += iHGo + iHBo
			break
	endswitch	
	
	imgHstr = "hist_" + imgroot	
	duplicate/O iHo, $imgHstr
	wave iH = $imgHstr
	// 16 bit rescale to step of 1
	if (bflag==1)
		SetScale/P x 0,1,"", iH
	endif
	
	imgHstr = "chist_" + imgroot
	duplicate/O iH, $imgHstr
	wave iCH = $imgHstr
	integrate/T iCH /D=iCH
	
	imgHstr = "dhist_" + imgroot
	duplicate/O iH, $imgHstr
	wave iDH = $imgHstr
	differentiate iDH /D=iDH
	WaveStats/Q iDH
	dhmax = max(abs(v_max),abs(v_min))
	iDH /= dhmax
	
#ifndef DEBUG
	killwaves/Z W_imageHist, W_imageHistR, W_imageHistG, W_imageHistB
#endif

	if (isstack)
		duplicate/O sstack_tmp wimg
	endif
	
	SetDataFolder cdf

	return 0
end

// generate an RoI mask
Function GenerateHistogramRoIMask(wimg,quiet)
	wave wimg
	variable quiet

	variable rtn = 0	
	variable ppx0, ppx1, ppy0, ppy1
	string maskName
	
	// get marquee from image graph	
	STRUCT S_MarqueeCoordinates mc
	Sf_GetMCCoordinates(mc)
	
	// no marquee or drawing for histogram
	if (!mc.hasmarquee && !mc.hasdrawing)
		if (!quiet)
			DoAlert/T="Restrict Histogram" 0, "No marquee region is present. Set a marquee region and try again."
		endif
		Checkbox checksethRoI_tabM, win=$k_fullpanel, value=0
		Checkbox checklockhRoI_tabM, win=$k_fullpanel, value=0, disable=2
		return rtn
	endif
	
	// put trunated pixel coordinates in local variables
	ppx0 = trunc(mc.pgleft)
	ppy0 = trunc(mc.pgtop)
	ppx1 = trunc(mc.pgright)
	ppy1 = trunc(mc.pgbottom)
	
	// store RoI values for the displayed image
	variable/G ihcpx0 = ppx0, ihcpx1 = ppx1, ihcpy0 = ppy0, ihcpy1 = ppy1
	maskName = NameofWave(wimg) + "_HRoIMask"
	duplicate/O wimg, $maskName
	wave imgMask = $maskName
	if (WaveDims(wimg) == 3)
		Redimension/B/U/N=(-1,-1,1) imgMask
	else
		Redimension/B/U imgMask
	endif
	imgMask = NaN
	imgMask[ppx0,ppx1][ppy0,ppy1] = 0
	
	rtn = 1
	
	return rtn

end

// scale the derivative of the histogram
Function ScaleDHistogram(sf)
	variable sf
	
	variable dhmin, dhmax
	dhmin = -(10^(-sf))
	dhmax = (10^(-sf))
	SetAxis/W=$k_fullhgraphs dhist dhmin,dhmax

	return 0	
end

// ***
// GENERAL PURPOSE

// set image note
Function TM_SetImageNote()

	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string rtstr = ""
	string wname = "root:" + cwdfolder + ":" + cwfile

	ControlInfo/W=$k_fullpanel setvarImgNote_tabM
	rtstr = s_value
	Note/K $wname
	Note $wname, rtstr
	
	// update contact print with new note
	ControlInfo/W=$k_fullpanel checkCP_tabM
	if (v_value)
		TM_StoreasContactPrint()
	endif

	return 0
end
